-- scope stuff
gearquipper = gearquipper or {};
local c = gearquipper;

GQ_OPTIONS = GQ_OPTIONS or {};
GQ_DATA = GQ_DATA or {};

local switching = false;
local GQ_VERSION = 7;

local OPT_VERSION = "OPT_VERSION";
c.OPT_ACTIONBARS = "OPT_ACTIONBARS";
c.OPT_NOTIFYCHANGES = "OPT_NOTIFYCHANGES";
c.OPT_HIGHLIGHTCHANGES = "OPT_HIGHLIGHTCHANGES";

-- init
function c:Init()
	c.realmName = GetRealmName();
	c.charName = UnitName("player");

	GQ_DATA[c.realmName] = GQ_DATA[c.realmName] or {};
	GQ_DATA[c.realmName][c.charName] = GQ_DATA[c.realmName][c.charName] or {};
	GQ_DATA[c.realmName][c.charName][c.FIELD_SETS] = GQ_DATA[c.realmName][c.charName][c.FIELD_SETS] or {};

	if GQ_OPTIONS[c.OPT_ACTIONBARS] == nil then
		GQ_OPTIONS[c.OPT_ACTIONBARS] = true;
	end

	if GQ_OPTIONS[c.OPT_NOTIFYCHANGES] == nil then
		GQ_OPTIONS[c.OPT_NOTIFYCHANGES] = true;
	end

	if GQ_OPTIONS[c.OPT_HIGHLIGHTCHANGES] == nil then
		GQ_OPTIONS[c.OPT_HIGHLIGHTCHANGES] = true;
	end

	c:InitUI();

	-- updates
	if not GQ_OPTIONS[OPT_VERSION] or tonumber(GQ_OPTIONS[OPT_VERSION]) < GQ_VERSION then
		if GQ_OPTIONS[OPT_VERSION] and tonumber(GQ_OPTIONS[OPT_VERSION]) < 2 then
			-- update if version < 2
			c:GetText("The version 2 update brought several new features, hence it needs the database to be reset. You will need to re-create your sets. I'm sorry and will try to avoid this in future updates.");
			GQ_DATA = {};
			GQ_OPTIONS = {};
			c:Init();
			return;
		end
		GQ_OPTIONS[OPT_VERSION] = GQ_VERSION;
	end

	c.initFinished = true;
	c:Println(c:GetText("AddOn loaded: Version: %s. Type /gq for options.", GQ_VERSION));
	c:Println(c:GetText("Visit https://www.curseforge.com/wow/addons/gearquipper for support, feedback and updates."));
end

-- command handling
SLASH_GEARQUIPPER1 = "/gq";

local CMD_ACTIONBARS = "actionbars";
local CMD_RESET = "reset";
local CMD_SWITCH = "switch";

SlashCmdList["GEARQUIPPER"] = function(msg)
	if msg then
		if c.initFinished then
			msg = c:Trim(msg);

			if msg == CMD_ACTIONBARS then
				GQ_OPTIONS[c.OPT_ACTIONBARS] = not GQ_OPTIONS[c.OPT_ACTIONBARS];
				c:Println(c:GetText("Action bars will be switched with your sets: %s", c:BoolToYesNo(GQ_OPTIONS[c.OPT_ACTIONBARS])));
				return;
			elseif msg == CMD_RESET then
				c:ShowResetDialog();
				return;
			elseif c:StartsWith(msg, CMD_SWITCH) then
				msg = c:Trim(msg:gsub(CMD_SWITCH, ""));
				if tContains(c:LoadSetNames(), msg) then
					c:SwitchToSet(msg);
				else
					c:Println(c:GetText("Macro failed: There is no set named \"%s\"."));
				end
				return;
			end

			c:Println(c:GetText("Unknown command. Possible parameters are: /gq ..."));
			c:Println(c:GetText("actionbars -> Toggles whether action bars are switched with your sets."));
			c:Println(c:GetText("reset -> Resets the GearQuipper addon (e.g. in case of errors)."));
			c:Println(c:GetText("switch [setname] -> Switches to the specified set. For macro use."));
		else
			c:Println(c:GetText("The AddOn has not been initialized due to an error. Please relog and try again."));
		end
	end
end

-- event handling
c.eventFrame = c.eventFrame or CreateFrame("Frame");
c.eventFrame:RegisterEvent("ADDON_LOADED");
c.eventFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
c.eventFrame:RegisterEvent("PLAYER_EQUIPMENT_CHANGED");
c.eventFrame:RegisterEvent("ACTIONBAR_SLOT_CHANGED");
c.eventFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");

c.eventFrame:RegisterEvent("ITEM_LOCKED");
c.eventFrame:RegisterEvent("ITEM_UNLOCKED");

c.eventFrame:HookScript("OnEvent", function(self, event, arg1, arg2, ...)
	if event then
		if event == "ADDON_LOADED" and arg1 == "GearQuipper" and not c.initFinished then
			c:Init();
		elseif event == "PLAYER_ENTERING_WORLD" then
			-- create default gear if not existing
			if table.getn(c:LoadSetNames()) == 0 then
				c:SaveDefaultSet();
			end
			c:SetPaperDollLabelText();
		elseif event == "PLAYER_EQUIPMENT_CHANGED" then --and _G["PaperDollFrame"]:IsVisible() then
			if not switching and c:GetSlotInfo()[arg1] then
				local currentSet = c:LoadCurrentSet();
				if currentSet then
					if not c:IsSetItem(arg1, currentSet[c.FIELD_SLOTS][arg1]) then
						c:SaveSlot(arg1);
						c:SetSlotInfo(arg1, "green");
					end
				end
			end
		elseif event == "ACTIONBAR_SLOT_CHANGED" then
			if not switching then
				c:SaveActionSlot(arg1);
			end
		elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
			if InCombatLockdown() then
				-- combat started
				c:DisablePaperDollButton();
			end
		elseif event == "ITEM_LOCKED" and not arg2 then
			local slotId = tonumber(arg1);
			if slotId >= 0 and slotId <= 19 then
				c:OpenQuickBar(slotId);
			end
		elseif event == "ITEM_UNLOCKED" then
			c:CloseQuickBars();
		end
	end
end);

function c:SwitchToSet(setName, retry)
	if (not switching or retry) and setName then
		local set = c:LoadSet(setName);
		if set and set[c.FIELD_SLOTS] then
			switching = true;

			for _, slotId in ipairs(c:GetSlotSwitchOrder()) do
				if not c:IsSetItem(slotId, set[c.FIELD_SLOTS][slotId])  then

					if InCombatLockdown() and not (slotId == INVSLOT_MAINHAND or slotId == INVSLOT_OFFHAND or slotId == INVSLOT_RANGED) then
						-- breaks loop if combat starts
						if not retry then
							c:Println(c:GetText("Switching interrupted by combat. It will be re-attempted once combat is over."));
						end
						C_Timer.After(1, function() c:SwitchToSet(setName, true); end);
						return;
					end

					if set[c.FIELD_SLOTS][slotId] == c.VALUE_NONE then
						ClearCursor();
						PickupInventoryItem(slotId);
						PutItemInBag(20);
						ClearCursor();
					elseif InCombatLockdown() and (slotId == INVSLOT_MAINHAND or slotId == INVSLOT_OFFHAND or slotId == INVSLOT_RANGED) then
						c:CallSecureAction(setName);
					else
						EquipItemByName(set[c.FIELD_SLOTS][slotId], slotId);
					end

					if GQ_OPTIONS[c.OPT_HIGHLIGHTCHANGES] then
						c:HighlightInventorySlot(slotId);
					end
				end
			end

			if not GQ_OPTIONS[c.OPT_ACTIONBARS] or c:LoadActionBars(setName, retry) then
				c:SaveCurrentSetName(setName);

				if _G["PaperDollFrame"]:IsVisible() then
					C_Timer.After(1, function()
						c:SetPaperDollLabelText();
						c:SetSlotInfo();
					end);
				end

				if retry then
					c:Println(c:GetText("Switch to set \"%s\" complete.", setName));
				end
				C_Timer.After(1, function() switching = false; end); -- workaround, todo: improve
			end
		end
	end
end

function c:IsSetComplete()
	local currentSet = c:LoadCurrentSet();
	if currentSet then
		for slotId, itemId in pairs(currentSet[c.FIELD_SLOTS]) do
			if not c:IsSetItem(slotId, itemId) then
				return false;
			end
		end
		return true;
	end
end

function c:IsSetItem(slotId, itemId)
	return c:GetItemId(slotId) == itemId;
end