-- scope stuff
gearquipper = gearquipper or {};
local c = gearquipper;

local SLOT_AMMO = "AmmoSlot";				--  0
local SLOT_HEAD = "HeadSlot";				--  1
local SLOT_NECK = "NeckSlot";				--  2
local SLOT_SHOULDER = "ShoulderSlot";		--  3
local SLOT_SHIRT = "ShirtSlot";				--  4
local SLOT_CHEST = "ChestSlot";				--  5
local SLOT_WAIST = "WaistSlot";				--  6
local SLOT_LEGS = "LegsSlot";				--  7
local SLOT_FEET = "FeetSlot";				--  8
local SLOT_WRIST = "WristSlot";				--  9
local SLOT_HANDS = "HandsSlot";				-- 10
local SLOT_FINGER1 = "Finger0Slot";			-- 11
local SLOT_FINGER2 = "Finger1Slot";			-- 12
local SLOT_TRINKET1 = "Trinket0Slot";		-- 13
local SLOT_TRINKET2 = "Trinket1Slot";		-- 14
local SLOT_BACK = "BackSlot";				-- 15
local SLOT_MAINHAND = "MainHandSlot";		-- 16
local SLOT_OFFHAND = "SecondaryHandSlot";	-- 17
local SLOT_RANGED = "RangedSlot";			-- 18
local SLOT_TABARD = "TabardSlot";			-- 19

local slotInfo = {
	[INVSLOT_AMMO] = SLOT_AMMO,
	[INVSLOT_HEAD] = SLOT_HEAD,
	[INVSLOT_NECK] = SLOT_NECK,
	[INVSLOT_SHOULDER] = SLOT_SHOULDER,
	[INVSLOT_BODY] = SLOT_SHIRT,
	[INVSLOT_CHEST] = SLOT_CHEST,
	[INVSLOT_WAIST] = SLOT_WAIST,
	[INVSLOT_LEGS] = SLOT_LEGS,
	[INVSLOT_FEET] = SLOT_FEET,
	[INVSLOT_WRIST] = SLOT_WRIST,
	[INVSLOT_HAND] = SLOT_HANDS,
	[INVSLOT_FINGER1] = SLOT_FINGER1,
	[INVSLOT_FINGER2] = SLOT_FINGER2,
	[INVSLOT_TRINKET1] = SLOT_TRINKET1,
	[INVSLOT_TRINKET2] = SLOT_TRINKET2,
	[INVSLOT_BACK] = SLOT_BACK,
	[INVSLOT_MAINHAND] = SLOT_MAINHAND,
	[INVSLOT_OFFHAND] = SLOT_OFFHAND,
	[INVSLOT_RANGED] = SLOT_RANGED,
	[INVSLOT_TABARD] = SLOT_TABARD
};

function c:GetSlotInfo()
	return slotInfo;
end

local slotSwitchOrder;
function c:GetSlotSwitchOrder()
	if not slotSwitchOrder then
		slotSwitchOrder = {};
		table.insert(slotSwitchOrder, INVSLOT_MAINHAND);
		table.insert(slotSwitchOrder, INVSLOT_OFFHAND);
		table.insert(slotSwitchOrder, INVSLOT_RANGED);

		for i = 0, 19 do
			if not tContains(slotSwitchOrder, i) then
				table.insert(slotSwitchOrder, i);
			end
		end
	end
	return slotSwitchOrder;
end

function c:GetSlotId(slotName)
	for slotId, name in pairs(c:GetSlotInfo()) do
		if name == slotName then
			return slotId;
		end
	end
end

local displaySlotNames = {
	[0] = INVTYPE_AMMO,
	[1] = INVTYPE_HEAD,
	[2] = INVTYPE_NECK,
	[3] = INVTYPE_SHOULDER,
	[4] = INVTYPE_BODY,
	[5] = INVTYPE_CHEST,
	[6] = INVTYPE_WAIST,
	[7] = INVTYPE_LEGS,
	[8] = INVTYPE_FEET,
	[9] = INVTYPE_WRIST,
	[10] = INVTYPE_HAND,
	[11] = INVTYPE_FINGER .. " 1",
	[12] = INVTYPE_FINGER .. " 2",
	[13] = INVTYPE_TRINKET .. " 1",
	[14] = INVTYPE_TRINKET .. " 2",
	[15] = INVTYPE_CLOAK,
	[16] = INVTYPE_WEAPONMAINHAND,
	[17] = INVTYPE_WEAPONOFFHAND,
	[18] = INVTYPE_RANGED,
	[19] = INVTYPE_ROBE
};
function c:GetDisplaySlotName(slotId)
	if displaySlotNames[slotId] then
		return displaySlotNames[slotId];
	end
end

function c:GetItemId(slotId)
	if slotId then
		local itemId = GetInventoryItemID("player", slotId);
		if itemId then
			return itemId;
		end
	end
	return c.VALUE_NONE;
end

local ITEMINFO_NAME = "ITEMINFO_NAME";
local ITEMINFO_LINK = "ITEMINFO_LINK";
local ITEMINFO_RARITY = "ITEMINFO_RARITY";
local ITEMINFO_LEVEL = "ITEMINFO_LEVEL";
local ITEMINFO_MINLEVEL = "ITEMINFO_MINLEVEL";
local ITEMINFO_TYPE = "ITEMINFO_TYPE";
local ITEMINFO_SUBTYPE = "ITEMINFO_SUBTYPE";
local ITEMINFO_STACKCOUNT = "ITEMINFO_STACKCOUNT";
local ITEMINFO_EQUIPLOC = "ITEMINFO_EQUIPLOC";
local function GetItemInfos(itemId, info)
	if itemId and itemId ~= c.VALUE_NONE then
		local itemInfo = {};
		itemInfo[ITEMINFO_NAME], itemInfo[ITEMINFO_LINK], itemInfo[ITEMINFO_RARITY], itemInfo[ITEMINFO_LEVEL],
		itemInfo[ITEMINFO_MINLEVEL], itemInfo[ITEMINFO_TYPE], itemInfo[ITEMINFO_SUBTYPE], itemInfo[ITEMINFO_STACKCOUNT],
		itemInfo[ITEMINFO_EQUIPLOC] = GetItemInfo(itemId);

		return itemInfo[info];
	end
end

function c:GetItemName(itemId)
	local itemName = GetItemInfos(itemId, ITEMINFO_NAME);
	if itemName then
		return itemName;
	end
	return c:GetText("Empty");
end

function c:GetItemLink(itemId)
	local itemLink = GetItemInfos(itemId, ITEMINFO_LINK);
	if itemLink then
		return itemLink;
	end
	return c:GetText("Empty");
end

function c:GetItemEquipLoc(itemId)
	return GetItemInfos(itemId, ITEMINFO_EQUIPLOC);
end

local RARITY_POOR = 0;
local RARITY_COMMON = 1;
local RARITY_UNCOMMON = 2;
local RARITY_RARE = 3;
local RARITY_EPIC = 4;
local RARITY_LEGENDARY = 5;
local RARITY_ARTIFACT = 6;
local RARITY_HEIRLOOM = 7;
local RARITY_WOWTOKEN = 8;
function c:GetItemRarity(itemId)
	local itemRarity = GetItemInfos(itemId, ITEMINFO_RARITY);
	if itemRarity then
		return itemRarity;
	end
	return RARITY_COMMON;
end

function c:GetSpellName(spellId)
	local spellName = GetSpellInfo(spellId);
	if spellName then
		return spellName;
	end
	return c:GetText("Empty");
end

function c:GetSpellRank(spellId)
	return GetSpellSubtext(spellId);
end

local MACROINFO_NAME = "MACROINFO_NAME";
local MACROINFO_TEXT = "MACROINFO_TEXT";
local function GetMacroInfos(macroSlotId, info)
	local macroInfo = {};
	macroInfo[MACROINFO_NAME], macroInfo["icon"], macroInfo[MACROINFO_TEXT], macroInfo["isLocal"] = GetMacroInfo(macroSlotId);
	if MACROINFO_NAME then
		return c:GetText("Macro \"%s\"", macroInfo[MACROINFO_NAME]);
	else
		return macroInfo[info];
	end
	return c:GetText("Empty");
end

function c:GetMacroName(macroSlotId)
	return GetMacroInfos(macroSlotId, MACROINFO_NAME);
end

function c:GetMacroText(macroSlotId)
	return GetMacroInfos(macroSlotId, MACROINFO_TEXT);
end

function c:GetRarityColor(rarity)
	return ITEM_QUALITY_COLORS[rarity];
end

c.ITEM_ID = "ITEM_ID";
c.ITEM_SLOT_ID = "ITEM_SLOT_ID";
c.ITEM_BAG_ID = "BAG_ID";
function c:GetMatchingItems(slotId)
	local equipLocs, result = {}, {};
	local equipLoc = c:GetItemEquipLoc(c:GetItemId(slotId));

	if slotId == INVSLOT_MAINHAND then
		table.insert(equipLocs, "INVTYPE_2HWEAPON");
		table.insert(equipLocs, "INVTYPE_WEAPON");
	elseif slotId == INVSLOT_OFFHAND then
		table.insert(equipLocs, "INVTYPE_WEAPON");
		table.insert(equipLocs, "INVTYPE_SHIELD");
	end

	if not tContains(equipLocs, equipLoc) then
		table.insert(equipLocs, equipLoc);
	end

	for bagId = 0, NUM_BAG_SLOTS do
		for slotId = 1, GetContainerNumSlots(bagId) do
			local itemId = GetContainerItemID(bagId, slotId);
			if itemId and tContains(equipLocs, c:GetItemEquipLoc(itemId)) then
				local newEntry = {};
				newEntry[c.ITEM_BAG_ID] = bagId;
				newEntry[c.ITEM_SLOT_ID] = slotId;
				newEntry[c.ITEM_ID] = itemId;
				table.insert(result, newEntry);
			end
		end
	end
	return result, table.getn(result);
end

function c:Println(str)
	if str then
		print("[GearQuipper] " .. str);
	end
end

function c:StartsWith(str, substr)
   return string.sub(str, 1, string.len(substr)) == substr;
end

function c:Trim(str)
	if str then
		return str:gsub("%s+", "");
	end
end

function c:BoolToYesNo(value)
	if value then
		return c:GetText("yes");
	end
	return c:GetText("no");
end

function c:GetTableSize(t)
    local count = 0
    for _, __ in pairs(t) do
        count = count + 1
    end
    return count
end

function c:Deepcopy(orig, copies)
    copies = copies or {}
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        if copies[orig] then
            copy = copies[orig]
        else
            copy = {}
            copies[orig] = copy
            setmetatable(copy, c:Deepcopy(getmetatable(orig), copies))
            for orig_key, orig_value in next, orig, nil do
                copy[c:Deepcopy(orig_key, copies)] = c:Deepcopy(orig_value, copies)
            end
        end
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

local actionBars = {"Action", "", "MultiBarRight", "MultiBarLeft", "MultiBarBottomRight", "MultiBarBottomLeft"};
function c:GetActionButton(slotId)
	local actionBar, buttonNo = math.floor((slotId - 1) / 12) + 1, ((slotId - 1) % 12) + 1;

	if _G[actionBars[actionBar] .. "Button" .. buttonNo] and _G[actionBars[actionBar] .. "Button" .. buttonNo]:IsVisible() then
		return _G[actionBars[actionBar] .. "Button" .. buttonNo];
	end
end